import logging

import requests

logger = logging.getLogger(__name__)


class ToucanClient:
    """
    Small client for sending request to a Toucan Toco back end.
    The constructor's mandatory parameter is the API base url. One can pass
    a small app name or a list of small app names, a token or an auth object
    for authentication.

    >>> # Example: Fetch etl config
    >>> client = ToucanClient('https://api.some.project.com')
    >>> small_app = client['my-small-app']
    >>> etl_config = small_app.config.etl.get()
    >>>
    >>> # Example: send a post request with some json data
    >>> response = small_app.config.etl.put(json={'DATA_SOURCE': ['example']})
    >>> # response.status_code equals 200 if everything went well
    """

    EXTRACTION_CACHE_PATH = 'extraction_cache'

    def __init__(self, base_route, **kwargs):
        # type: (str) -> SmallAppRequester
        self.__dict__['_path'] = []
        self.__dict__['kwargs'] = kwargs
        self.__dict__['stage'] = ''
        self.__dict__['_dfs'] = None
        self.__dict__['_cache_path'] = None

        self.__dict__['base_route'] = base_route
        if base_route.endswith('/'):
            self.__dict__['base_route'] = base_route[:-1]

    @property
    def method(self):
        # type: () -> str
        return self._path[-1]

    @property
    def options(self):
        # type: () -> str
        if self.stage:
            return '?stage={}'.format(self.stage)
        return ''

    @property
    def route(self):
        # type: () -> str
        route = '/'.join([self.base_route] + self._path[:-1])
        route += self.options

        self.__dict__['_path'] = []
        return route

    def __getattr__(self, key):
        self._path.append(key)
        return self

    def __setattr__(self, key, value):
        if key == 'stage':
            self.__dict__['stage'] = value
        else:
            self.kwargs[key] = value

    def __call__(self):
        # type: () -> requests.Response
        method, route = self.method, self.route
        func = getattr(requests, method)
        return func(route, **self.kwargs)



import pytest

from toucan_client.client import ToucanClient

SMALL_APP_NAME = 'my-small-app'

BASE_ROUTE = 'fake.route/my-small-app'
BASE_ROUTE_2 = 'fake.route/my-small-app/'


@pytest.fixture(name='small_app', scope='function')
def gen_small_app():
    return ToucanClient(BASE_ROUTE)


def test_simple_get(small_app):
    """It should build the right URL"""
    _ = small_app.config.etl.get
    assert small_app.method == 'get'
    assert small_app.route == '{}/config/etl'.format(BASE_ROUTE)


def test_simple_get_with_baseroute_2():
    small_app = ToucanClient(BASE_ROUTE_2)
    _ = small_app.config.etl.get
    assert small_app.method == 'get'
    assert small_app.route == '{}/config/etl'.format(BASE_ROUTE)


def test_simple_get_with_stage(small_app):
    """It should build the right URL (with staging stage)"""
    small_app.stage = 'staging'
    _ = small_app.config.etl.get
    assert small_app.method == 'get'
    assert small_app.route == '{}/config/etl?stage=staging'.format(BASE_ROUTE)


def test_call(small_app, mocker):
    mock_getattr = mocker.patch('requests.get')
    mock_getattr.return_value = 1
    mock_getattr.return_value = 1
    res = small_app.config.etl.get()
    assert res == 1


def test_kwargs(small_app):
    """It should add kwargs"""
    small_app.best_character = 'Ryu'
    _ = small_app.config.etl.get
    assert small_app.kwargs == {'best_character': 'Ryu'}
